import os

DATADIR = os.path.abspath('tests/data')
FILES = ['coutwildrnp.shp', 'coutwildrnp.shx', 'coutwildrnp.dbf', 'coutwildrnp.prj']

def create_zipfile(zipfilename):
    import zipfile
    with zipfile.ZipFile(zipfilename, 'w') as zip:
        for filename in FILES:
            zip.write(os.path.join(DATADIR, filename), filename)

def create_tarfile(tarfilename):
    import tarfile
    with tarfile.open(tarfilename, 'w') as tar:
        for filename in FILES:
            tar.add(os.path.join(DATADIR, filename), arcname='testing/%s' % filename)

def create_jsonfile(jsonfilename):
    import json
    import fiona
    with fiona.open(os.path.join(DATADIR, FILES[0]), 'r') as source:
        features = [feat for feat in source]
    my_layer = {'type': 'FeatureCollection',
                'features': features}
    with open(jsonfilename, 'w') as f:
        f.write(json.dumps(my_layer))

def setup():
    """Setup function for nosetests to create test files if they do not exist
    """
    zipfile = os.path.join(DATADIR, 'coutwildrnp.zip')
    tarfile = os.path.join(DATADIR, 'coutwildrnp.tar')
    jsonfile = os.path.join(DATADIR, 'coutwildrnp.json')
    if not os.path.exists(zipfile):
        create_zipfile(zipfile)
    if not os.path.exists(tarfile):
        create_tarfile(tarfile)
    if not os.path.exists(jsonfile):
        create_jsonfile(jsonfile)



import collections
from six.moves import UserDict
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict


# Users can pass in objects that subclass a few different objects
# More specifically, rasterio has a CRS() class that subclasses UserDict()
# In Python 2 UserDict() is in its own module and does not subclass Mapping()
DICT_TYPES = (dict, collections.Mapping, UserDict)



"""$ fiona ls"""


import json

import click
from cligj import indent_opt

import fiona


@click.command()
@click.argument('input', type=click.Path(exists=True))
@indent_opt
@click.pass_context
def ls(ctx, input, indent):

    """
    List layers in a datasource.
    """

    verbosity = (ctx.obj and ctx.obj['verbosity']) or 2

    with fiona.drivers(CPL_DEBUG=verbosity > 2):
        result = fiona.listlayers(input)
        click.echo(json.dumps(result, indent=indent))



import logging
import sys

import fiona

logging.basicConfig(stream=sys.stderr, level=logging.INFO)

def test_collection_get():
    with fiona.open('tests/data/coutwildrnp.shp') as src:
        result = src[5]
        assert result['id'] == '5'

def test_collection_slice():
    with fiona.open('tests/data/coutwildrnp.shp') as src:
        results = src[:5]
        assert isinstance(results, list)
        assert len(results) == 5
        assert results[4]['id'] == '4'

def test_collection_iterator_slice():
    with fiona.open('tests/data/coutwildrnp.shp') as src:
        results = list(src.items(5))
        assert len(results) == 5
        k, v = results[4]
        assert k == 4
        assert v['id'] == '4'

def test_collection_iterator_next():
    with fiona.open('tests/data/coutwildrnp.shp') as src:
        k, v = next(src.items(5, None))
        assert k == 5
        assert v['id'] == '5'

def test_collection_iterator_items_slice():

    with fiona.open('tests/data/coutwildrnp.shp') as src:
        l = len(src)

        items = list(src.items(0, 5))
        assert len(items) == 5

        items = list(src.items(1, 5))
        assert len(items) == 4

        items = list(src.items(-5, None))
        assert len(items) == 5

        items = list(src.items(-5, -1))
        assert len(items) == 4

        items = list(src.items(0, None))
        assert len(items) == l

        items = list(src.items(5, None))
        assert len(items) == (l - 5)

        items = list(src.items(5, None, -1))
        assert len(items) == 6

        items = list(src.items(5, None, -2))
        assert len(items) == 3

        items = list(src.items(4, None, -2))
        assert len(items) == 3

        items = list(src.items(-1, -5, -1))
        assert len(items) == 4

        items = list(src.items(-5, None, -1))
        assert len(items) == (l - 5 + 1)

def test_collection_iterator_keys_next():
    with fiona.open('tests/data/coutwildrnp.shp') as src:
        k = next(src.keys(5, None))
        assert k == 5



"""$ fio insp"""


import code
import logging
import sys

import click

import fiona


@click.command(short_help="Open a dataset and start an interpreter.")
@click.argument('src_path', type=click.Path(exists=True))
@click.option('--ipython', 'interpreter', flag_value='ipython',
              help="Use IPython as interpreter.")
@click.pass_context
def insp(ctx, src_path, interpreter):

    verbosity = (ctx.obj and ctx.obj['verbosity']) or 2
    logger = logging.getLogger('fio')

    banner = 'Fiona %s Interactive Inspector (Python %s)\n' \
             'Type "src.schema", "next(src)", or "help(src)" ' \
             'for more information.' \
             % (fiona.__version__, '.'.join(map(str, sys.version_info[:3])))

    try:
        with fiona.drivers(CPL_DEBUG=verbosity > 2):
            with fiona.open(src_path) as src:

                scope = locals()

                if not interpreter:
                    code.interact(banner, local=scope)
                elif interpreter == 'ipython':
                    import IPython
                    IPython.InteractiveShell.banner1 = banner
                    IPython.start_ipython(argv=[], user_ns=scope)
                else:
                    raise click.ClickException(
                        'Interpreter {} is unsupported or missing '
                        'dependencies'.format(interpreter))
    except Exception:
        logger.exception("Exception caught during processing")
        raise click.Abort()



from distutils.command.sdist import sdist
from distutils import log
import logging
import os
import shutil
import subprocess
import sys

from setuptools import setup
from setuptools.extension import Extension


# Use Cython if available.
try:
    from Cython.Build import cythonize
except ImportError:
    cythonize = None


def check_output(cmd):
    # since subprocess.check_output doesn't exist in 2.6
    # we wrap it here.
    try:
        out = subprocess.check_output(cmd)
        return out.decode('utf')
    except AttributeError:
        # For some reasone check_output doesn't exist
        # So fall back on Popen
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        out, err = p.communicate()
        return out


def copy_data_tree(datadir, destdir):
    try:
        shutil.rmtree(destdir)
    except OSError:
        pass
    shutil.copytree(datadir, destdir)

# Parse the version from the fiona module.
with open('fiona/__init__.py', 'r') as f:
    for line in f:
        if line.find("__version__") >= 0:
            version = line.split("=")[1].strip()
            version = version.strip('"')
            version = version.strip("'")
            break

# Fiona's auxiliary files are UTF-8 encoded and we'll specify this when
# reading with Python 3+
open_kwds = {}
if sys.version_info > (3,):
    open_kwds['encoding'] = 'utf-8'

with open('VERSION.txt', 'w', **open_kwds) as f:
    f.write(version)

with open('README.rst', **open_kwds) as f:
    readme = f.read()

with open('CREDITS.txt', **open_kwds) as f:
    credits = f.read()

with open('CHANGES.txt', **open_kwds) as f:
    changes = f.read()

# Set a flag for builds where the source directory is a repo checkout.
source_is_repo = os.path.exists("MANIFEST.in")


# Extend distutil's sdist command to generate C extension sources from
# both `ogrext`.pyx` and `ogrext2.pyx` for GDAL 1.x and 2.x.
class sdist_multi_gdal(sdist):
    def run(self):
        shutil.copy('fiona/ogrext1.pyx', 'fiona/ogrext.pyx')
        _ = check_output(['cython', '-v', '-f', 'fiona/ogrext.pyx',
                          '-o', 'fiona/ogrext1.c'])
        print(_)
        shutil.copy('fiona/ogrext2.pyx', 'fiona/ogrext.pyx')
        _ = check_output(['cython', '-v', '-f', 'fiona/ogrext.pyx',
                          '-o', 'fiona/ogrext2.c'])
        print(_)
        sdist.run(self)

# Building Fiona requires options that can be obtained from GDAL's gdal-config
# program or can be specified using setup arguments. The latter override the
# former.
#
# A GDAL API version is strictly required. Without this the setup script
# cannot know whether to use the GDAL version 1 or 2 source files. The GDAL
# API version can be specified in 2 ways.
#
# 1. By the gdal-config program, optionally pointed to by GDAL_CONFIG
# 2. By a GDAL_VERSION environment variable. This overrides number 1.

include_dirs = []
library_dirs = []
libraries = []
extra_link_args = []
gdal_output = [None for i in range(4)]
gdalversion = '2'

if 'clean' not in sys.argv:
    try:
        gdal_config = os.environ.get('GDAL_CONFIG', 'gdal-config')
        for i, flag in enumerate(
                ["--cflags", "--libs", "--datadir", "--version"]):
            gdal_output[i] = check_output([gdal_config, flag]).strip()
        for item in gdal_output[0].split():
            if item.startswith("-I"):
                include_dirs.extend(item[2:].split(":"))
        for item in gdal_output[1].split():
            if item.startswith("-L"):
                library_dirs.extend(item[2:].split(":"))
            elif item.startswith("-l"):
                libraries.append(item[2:])
            else:
                # e.g. -framework GDAL
                extra_link_args.append(item)
        gdalversion = gdal_output[3]
        if gdalversion:
            log.info("GDAL API version obtained from gdal-config: %s",
                     gdalversion)

    except Exception as e:
        if os.name == "nt":
            log.info("Building on Windows requires extra options to setup.py "
                     "to locate needed GDAL files.\nMore information is "
                     "available in the README.")
        else:
            log.warn("Failed to get options via gdal-config: %s", str(e))

    # Get GDAL API version from environment variable.
    if 'GDAL_VERSION' in os.environ:
        gdalversion = os.environ['GDAL_VERSION']
        log.info("GDAL API version obtained from environment: %s", gdalversion)

    # Get GDAL API version from the command line if specified there.
    if '--gdalversion' in sys.argv:
        index = sys.argv.index('--gdalversion')
        sys.argv.pop(index)
        gdalversion = sys.argv.pop(index)
        log.info("GDAL API version obtained from command line option: %s",
                 gdalversion)

    if not gdalversion:
        log.fatal("A GDAL API version must be specified. Provide a path "
                  "to gdal-config using a GDAL_CONFIG environment variable "
                  "or use a GDAL_VERSION environment variable.")
        sys.exit(1)

    if os.environ.get('PACKAGE_DATA'):
        destdir = 'fiona/gdal_data'
        if gdal_output[2]:
            log.info("Copying gdal data from %s" % gdal_output[2])
            copy_data_tree(gdal_output[2], destdir)
        else:
            # check to see if GDAL_DATA is defined
            gdal_data = os.environ.get('GDAL_DATA', None)
            if gdal_data:
                log.info("Copying gdal data from %s" % gdal_data)
                copy_data_tree(gdal_data, destdir)

        # Conditionally copy PROJ.4 data.
        projdatadir = os.environ.get('PROJ_LIB', '/usr/local/share/proj')
        if os.path.exists(projdatadir):
            log.info("Copying proj data from %s" % projdatadir)
            copy_data_tree(projdatadir, 'fiona/proj_data')

ext_options = dict(
    include_dirs=include_dirs,
    library_dirs=library_dirs,
    libraries=libraries,
    extra_link_args=extra_link_args)

# Define the extension modules.
ext_modules = []

if source_is_repo and "clean" not in sys.argv:
    # When building from a repo, Cython is required.
    log.info("MANIFEST.in found, presume a repo, cythonizing...")
    if not cythonize:
        log.fatal("Cython.Build.cythonize not found. "
                  "Cython is required to build from a repo.")
        sys.exit(1)

    if gdalversion.startswith("1"):
        log.info("Building Fiona for gdal 1.x: {0}".format(gdalversion))
        shutil.copy('fiona/ogrext1.pyx', 'fiona/ogrext.pyx')
    else:
        log.info("Building Fiona for gdal 2.x: {0}".format(gdalversion))
        shutil.copy('fiona/ogrext2.pyx', 'fiona/ogrext.pyx')

    ext_modules = cythonize([
        Extension('fiona._geometry', ['fiona/_geometry.pyx'], **ext_options),
        Extension('fiona._transform', ['fiona/_transform.pyx'], **ext_options),
        Extension('fiona._crs', ['fiona/_crs.pyx'], **ext_options),
        Extension('fiona._drivers', ['fiona/_drivers.pyx'], **ext_options),
        Extension('fiona._err', ['fiona/_err.pyx'], **ext_options),
        Extension('fiona.ogrext', ['fiona/ogrext.pyx'], **ext_options)])

# If there's no manifest template, as in an sdist, we just specify .c files.
elif "clean" not in sys.argv:
    ext_modules = [
        Extension('fiona._transform', ['fiona/_transform.cpp'], **ext_options),
        Extension('fiona._geometry', ['fiona/_geometry.c'], **ext_options),
        Extension('fiona._crs', ['fiona/_crs.c'], **ext_options),
        Extension('fiona._drivers', ['fiona/_drivers.c'], **ext_options),
        Extension('fiona._err', ['fiona/_err.c'], **ext_options)]

    if gdalversion.startswith("1"):
        log.info("Building Fiona for gdal 1.x: {0}".format(gdalversion))
        ext_modules.append(
            Extension('fiona.ogrext', ['fiona/ogrext1.c'], **ext_options))
    else:
        log.info("Building Fiona for gdal 2.x: {0}".format(gdalversion))
        ext_modules.append(
            Extension('fiona.ogrext', ['fiona/ogrext2.c'], **ext_options))

requirements = [
    'cligj>=0.4',
    'click-plugins',
    'six>=1.7',
    'munch']

if sys.version_info < (2, 7):
    requirements.append('argparse')
    requirements.append('ordereddict')

if sys.version_info < (3, 4):
    requirements.append('enum34')

setup_args = dict(
    cmdclass={'sdist': sdist_multi_gdal},
    metadata_version='1.2',
    name='Fiona',
    version=version,
    requires_python='>=2.6',
    requires_external='GDAL (>=1.8)',
    description="Fiona reads and writes spatial data files",
    license='BSD',
    keywords='gis vector feature data',
    author='Sean Gillies',
    author_email='sean.gillies@gmail.com',
    maintainer='Sean Gillies',
    maintainer_email='sean.gillies@gmail.com',
    url='http://github.com/Toblerity/Fiona',
    long_description=readme + "\n" + changes + "\n" + credits,
    package_dir={'': '.'},
    packages=['fiona', 'fiona.fio'],
    entry_points='''
        [console_scripts]
        fio=fiona.fio.main:main_group

        [fiona.fio_commands]
        bounds=fiona.fio.bounds:bounds
        calc=fiona.fio.calc:calc
        cat=fiona.fio.cat:cat
        collect=fiona.fio.collect:collect
        distrib=fiona.fio.distrib:distrib
        dump=fiona.fio.dump:dump
        env=fiona.fio.env:env
        filter=fiona.fio.filter:filter
        info=fiona.fio.info:info
        insp=fiona.fio.insp:insp
        load=fiona.fio.load:load
        ls=fiona.fio.ls:ls
        ''',
    install_requires=requirements,
    extras_require={
        'calc': ['shapely'],
        'test': ['nose']},
    tests_require=['nose'],
    test_suite='nose.collector',
    ext_modules=ext_modules,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering :: GIS'])

if os.environ.get('PACKAGE_DATA'):
    setup_args['package_data'] = {'fiona': ['gdal_data/*', 'proj_data/*']}

setup(**setup_args)



import os
import shutil
import tempfile
import unittest

import fiona
from fiona.errors import UnsupportedGeometryTypeError


class SchemaOrder(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_schema_ordering_items(self):
        items = [('title', 'str:80'), ('date', 'date')]
        with fiona.open(os.path.join(self.tempdir, 'test_schema.shp'), 'w',
                driver="ESRI Shapefile",
                schema={
                    'geometry': 'LineString', 
                    'properties': items }) as c:
            self.assertEqual(list(c.schema['properties'].items()), items)
        with fiona.open(os.path.join(self.tempdir, 'test_schema.shp')) as c:
            self.assertEqual(list(c.schema['properties'].items()), items)

class ShapefileSchema(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_schema(self):
        items = sorted({
            'AWATER10': 'float',
            'CLASSFP10': 'str',
            'ZipCodeType': 'str',
            'EstimatedPopulation': 'float',
            'LocationType': 'str',
            'ALAND10': 'float',
            'TotalWages': 'float',
            'FUNCSTAT10': 'str',
            'Long': 'float',
            'City': 'str',
            'TaxReturnsFiled': 'float',
            'State': 'str',
            'Location': 'str',
            'GSrchCnt': 'float',
            'INTPTLAT10': 'str',
            'Lat': 'float',
            'MTFCC10': 'str',
            'Decommisioned': 'str',
            'GEOID10': 'str',
            'INTPTLON10': 'str'}.items())
        with fiona.open(os.path.join(self.tempdir, 'test_schema.shp'), 'w',
                driver="ESRI Shapefile",
                schema={
                    'geometry': 'Polygon', 
                    'properties': items }) as c:
            self.assertEqual(list(c.schema['properties'].items()), items)
            c.write(
                {'geometry': {'coordinates': [[(-117.882442, 33.783633),
                                               (-117.882284, 33.783817),
                                               (-117.863348, 33.760016),
                                               (-117.863478, 33.760016),
                                               (-117.863869, 33.760017),
                                                (-117.864, 33.760017999999995),
                                                (-117.864239, 33.760019),
                                                (-117.876608, 33.755769),
                                                (-117.882886, 33.783114),
                                                (-117.882688, 33.783345),
                                                (-117.882639, 33.783401999999995),
                                                (-117.88259, 33.78346),
                                                (-117.882442, 33.783633)]],
                               'type': 'Polygon'},
                 'id': '1',
                 'properties':{
                    'ALAND10': 8819240.0,
                    'AWATER10': 309767.0,
                    'CLASSFP10': 'B5',
                    'City': 'SANTA ANA',
                    'Decommisioned': False,
                    'EstimatedPopulation': 27773.0,
                    'FUNCSTAT10': 'S',
                    'GEOID10': '92706',
                    'GSrchCnt': 0.0,
                    'INTPTLAT10': '+33.7653010',
                    'INTPTLON10': '-117.8819759',
                    'Lat': 33.759999999999998,
                    'Location': 'NA-US-CA-SANTA ANA',
                    'LocationType': 'PRIMARY',
                    'Long': -117.88,
                    'MTFCC10': 'G6350',
                    'State': 'CA',
                    'TaxReturnsFiled': 14635.0,
                    'TotalWages': 521280485.0,
                    'ZipCodeType': 'STANDARD'},
                 'type': 'Feature'} )
            self.assertEqual(len(c), 1)
        with fiona.open(os.path.join(self.tempdir, 'test_schema.shp')) as c:
            self.assertEqual(
                list(c.schema['properties'].items()), 
                sorted([('AWATER10', 'float:24.15'), 
                 ('CLASSFP10', 'str:80'), 
                 ('ZipCodeTyp', 'str:80'), 
                 ('EstimatedP', 'float:24.15'), 
                 ('LocationTy', 'str:80'), 
                 ('ALAND10', 'float:24.15'), 
                 ('INTPTLAT10', 'str:80'), 
                 ('FUNCSTAT10', 'str:80'), 
                 ('Long', 'float:24.15'), 
                 ('City', 'str:80'), 
                 ('TaxReturns', 'float:24.15'), 
                 ('State', 'str:80'), 
                 ('Location', 'str:80'), 
                 ('GSrchCnt', 'float:24.15'), 
                 ('TotalWages', 'float:24.15'), 
                 ('Lat', 'float:24.15'), 
                 ('MTFCC10', 'str:80'), 
                 ('INTPTLON10', 'str:80'), 
                 ('GEOID10', 'str:80'), 
                 ('Decommisio', 'str:80')]) )
            f = next(iter(c))
            self.assertEqual(f['properties']['EstimatedP'], 27773.0)


class FieldTruncationTestCase(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_issue177(self):
        name = os.path.join(self.tempdir, 'output.shp')

        kwargs = {
            'driver': 'ESRI Shapefile',
            'crs': 'EPSG:4326',
            'schema': {
                'geometry': 'Point',
                'properties': [('a_fieldname', 'float')]}}

        with fiona.open(name, 'w', **kwargs) as dst:
            rec = {}
            rec['geometry'] = {'type': 'Point', 'coordinates': (0, 0)}
            rec['properties'] = {'a_fieldname': 3.0}
            dst.write(rec)

        with fiona.open(name) as src:
            first = next(iter(src))
            assert first['geometry'] == {'type': 'Point', 'coordinates': (0, 0)}
            assert first['properties']['a_fieldnam'] == 3.0


def test_unsupported_geometry_type():
    tmpdir = tempfile.mkdtemp()
    tmpfile = os.path.join(tmpdir, 'test-test-geom.shp')

    profile = {
        'driver': 'ESRI Shapefile',
        'schema': {
            'geometry': 'BOGUS',
            'properties': {}}}

    try:
        fiona.open(tmpfile, 'w', **profile)
    except UnsupportedGeometryTypeError:
        assert True
